using System;
using System.IO;
using BeeDevelopment.Brazil;
using Stetson.Emulator.Hardware;

namespace Stetson.Emulator {
	public partial class TI83Plus : Z80A, IDisposable {

		private const int PageSize = 0x4000;

		public TI83Plus(string romFile) {

			this.linkPort = new IOPort(this);
			this.lcdDriver = new T6A04(this);
			this.keypad = new Keypad();

			byte[] RomFile = File.ReadAllBytes(romFile);
			this.RomPages = new byte[32][];
			for (int i = 0; i < this.RomPages.Length; ++i) {
				this.RomPages[i] = new byte[PageSize];
				Array.Copy(RomFile, i * PageSize, this.RomPages[i], 0, PageSize);
			}

			this.RamPages = new byte[2][];
			for (int i = 0; i < this.RamPages.Length; ++i) this.RamPages[i] = new byte[PageSize];

			this.WriteMemoryPageA(0);
			this.WriteMemoryPageB(0);
			this.WriteMemoryMapControl(0);
		}

		public void Dispose() {
			if (this.lcdDriver != null) this.lcdDriver.Dispose();
		}

		int CyclesToRun = 0;
		const int CpuCyclesPerTimerTick = 183;

		int Timer1Pending = 0;
		int Timer2Pending = 0;

		public new void FetchExecute(int cycles) {

			CyclesToRun += cycles;
			while (CyclesToRun >= 0) {

				CyclesToRun -= CpuCyclesPerTimerTick;

				this.RunTimerInterrupts();

				base.FetchExecute(CpuCyclesPerTimerTick);

			}
		}



	}
}